@@ -22,13 +22,13 @@ cm_control_points = res.x[:np.prod(p['cm_shape'])].reshape(p['cm_shape'])
 rvecs = res.x[np.prod(p['cm_shape']):][:n_images * 3].reshape((n_images, 3, 1))
 tvecs = res.x[np.prod(p['cm_shape']) + n_images * 3:].reshape((n_images, 3, 1))
 
-ba = BundleAdjustment(p, obj_points, rvecs, tvecs, all_corners_2D, cameraMatrix, distCoeffs, cm_control_points)
+ba = BundleAdjustment(obj_points, rvecs, tvecs, all_corners_2D, cameraMatrix, distCoeffs, cm_control_points)
 
 ls_params = np.hstack((cm_control_points.ravel(), rvecs.ravel(), tvecs.ravel()))
 
 #residuals_3D = ba.calc_residuals_3D(ls_params, p)
 
-residuals_2D, estimated_points_2D, correct_points_2D = ba.calc_residuals_2D(ls_params, p, return_points_2D=True)
+residuals_2D, estimated_points_2D, correct_points_2D = ba.calc_residuals_2D(ls_params, return_points_2D=True)
 
 np.save('residuals_and_points_2D_2020-05-09_22-32-50', [residuals_2D, estimated_points_2D, correct_points_2D])
 
